@@ -106,16 +106,20 @@ type InstallManager struct {
 	readClusterMetadata              func(*hivev1.ClusterProvision, *InstallManager) ([]byte, *installertypes.ClusterMetadata, error)
 	uploadAdminKubeconfig            func(*hivev1.ClusterProvision, *InstallManager) (*corev1.Secret, error)
 	uploadAdminPassword              func(*hivev1.ClusterProvision, *InstallManager) (*corev1.Secret, error)
+	loadAdminPassword                func(*InstallManager) (string, error)
+	provisionCluster                 func(*InstallManager) error
 	readInstallerLog                 func(*hivev1.ClusterProvision, *InstallManager, bool) (string, error)
 	waitForProvisioningStage         func(*hivev1.ClusterProvision, *InstallManager) error
-	isGatherLogsEnabled              func() bool
 	waitForInstallCompleteExecutions int
 	binaryDir                        string
+	actuator                         LogUploaderActuator
 }
 
 // NewInstallManagerCommand is the entrypoint to create the 'install-manager' subcommand
 func NewInstallManagerCommand() *cobra.Command {
-	im := &InstallManager{}
+	im := &InstallManager{
+		actuator: getActuator(),
+	}
 	cmd := &cobra.Command{
 		Use:   "install-manager NAMESPACE CLUSTER_PROVISION_NAME",
 		Short: "Executes and oversees the openshift-installer.",
@@ -167,9 +171,10 @@ func (m *InstallManager) Complete(args []string) error {
 	m.readClusterMetadata = readClusterMetadata
 	m.uploadAdminKubeconfig = uploadAdminKubeconfig
 	m.uploadAdminPassword = uploadAdminPassword
+	m.loadAdminPassword = loadAdminPassword
 	m.readInstallerLog = readInstallerLog
-	m.isGatherLogsEnabled = isGatherLogsEnabled
 	m.cleanupFailedProvision = cleanupFailedProvision
+	m.provisionCluster = provisionCluster
 	m.waitForProvisioningStage = waitForProvisioningStage
 
 	// Set log level
@@ -200,6 +205,16 @@ func (m *InstallManager) Complete(args []string) error {
 		m.log.WithField("workdir", m.WorkDir).Fatalf("workdir does not exist")
 	}
 
+	// Swap some function implementations if this is flagged to be a fake install:
+	fakeInstall, err := strconv.ParseBool(os.Getenv(constants.FakeClusterInstallEnvVar))
+	if fakeInstall {
+		m.log.Warnf("%s set to true, swapping function implementations to fake an installation",
+			constants.FakeClusterInstallEnvVar)
+		m.loadAdminPassword = fakeLoadAdminPassword
+		m.readClusterMetadata = fakeReadClusterMetadata
+		m.provisionCluster = fakeProvisionCluster
+	}
+
 	return nil
 }
 
@@ -328,6 +343,7 @@ func (m *InstallManager) Run() error {
 			m.log.WithError(readErr).Error("error reading asset generation log")
 			return err
 		}
+
 		m.log.Info("updating clusterprovision")
 		if err := m.updateClusterProvision(
 			provision,
@@ -370,6 +386,7 @@ func (m *InstallManager) Run() error {
 			provision.Spec.Metadata = &runtime.RawExtension{Raw: metadataBytes}
 			provision.Spec.InfraID = pointer.StringPtr(metadata.InfraID)
 			provision.Spec.ClusterID = pointer.StringPtr(metadata.ClusterID)
+
 			provision.Spec.AdminKubeconfigSecretRef = &corev1.LocalObjectReference{
 				Name: kubeconfigSecret.Name,
 			}
@@ -398,7 +415,7 @@ func (m *InstallManager) Run() error {
 		}
 	}
 
-	installErr := m.provisionCluster()
+	installErr := m.provisionCluster(m)
 	if installErr != nil {
 		m.log.WithError(installErr).Error("error running openshift-install, running deprovision to clean up")
 
@@ -415,17 +432,10 @@ func (m *InstallManager) Run() error {
 		}
 
 		// Fetch logs from all cluster machines:
-		if m.isGatherLogsEnabled() {
-			m.gatherLogs(cd, sshKeyPath, sshAgentSetupErr)
-		}
-
-		// TODO: should we timebox this deprovision attempt in the event it gets stuck?
-		if err := m.cleanupFailedInstall(cd, provision); err != nil {
-			// Log the error but continue. It is possible we were not able to clear the infraID
-			// here, but we will attempt this again anyhow when the next job retries. The
-			// goal here is just to minimize running resources in the event of a long wait
-			// until the next retry.
-			m.log.WithError(err).Error("error while trying to deprovision after failed install")
+		if m.actuator == nil {
+			m.log.Debug("Unable to find log storage actuator. Disabling gathering logs.")
+		} else {
+			m.gatherLogs(provision, cd, sshKeyPath, sshAgentSetupErr)
 		}
 	}
 
@@ -453,6 +463,20 @@ func (m *InstallManager) Run() error {
 	return nil
 }
 
+func getActuator() LogUploaderActuator {
+	// As we add more LogUploaderActuators, add them here
+	actuators := []LogUploaderActuator{
+		&s3LogUploaderActuator{awsClientFn: getAWSClient},
+	}
+
+	for _, a := range actuators {
+		if a.IsConfigured() {
+			return a
+		}
+	}
+	return nil
+}
+
 func (m *InstallManager) waitForFiles(files []string) {
 	m.log.Infof("waiting for files to be available: %v", files)
 
@@ -642,7 +666,6 @@ func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeploymen
 // generateAssets runs openshift-install commands to generate on-disk assets we need to
 // upload or modify prior to provisioning resources in the cloud.
 func (m *InstallManager) generateAssets(provision *hivev1.ClusterProvision) error {
-
 	m.log.Info("running openshift-install create manifests")
 	err := m.runOpenShiftInstallCommand("create", "manifests")
 	if err != nil {
@@ -667,13 +690,14 @@ func (m *InstallManager) generateAssets(provision *hivev1.ClusterProvision) erro
 		m.log.WithError(err).Error("error generating installer assets")
 		return err
 	}
+
 	m.log.Info("assets generated successfully")
 	return nil
 }
 
 // provisionCluster invokes the openshift-install create cluster command to provision resources
 // in the cloud.
-func (m *InstallManager) provisionCluster() error {
+func provisionCluster(m *InstallManager) error {
 
 	m.log.Info("running openshift-install create cluster")
 
@@ -690,6 +714,7 @@ func (m *InstallManager) provisionCluster() error {
 			return err
 		}
 	}
+
 	return nil
 }
 
@@ -826,12 +851,6 @@ func (m *InstallManager) loadClusterDeployment(provision *hivev1.ClusterProvisio
 	return cd, nil
 }
 
-func isGatherLogsEnabled() bool {
-	// By default we assume to gather logs, only disable if explicitly told to via HiveConfig.
-	envVarValue := os.Getenv(constants.SkipGatherLogsEnvVar)
-	return envVarValue != "true"
-}
-
 // gatherLogs will attempt to gather logs after a failed install. First we attempt
 // to gather logs from the bootstrap node. If this fails, we may have made it far enough
 // to teardown the bootstrap node, in which case we then attempt to gather with
@@ -839,7 +858,7 @@ func isGatherLogsEnabled() bool {
 // If neither succeeds we do not consider this a fatal error,
 // we're just gathering as much information as we can and then proceeding with cleanup
 // so we can re-try.
-func (m *InstallManager) gatherLogs(cd *hivev1.ClusterDeployment, sshPrivKeyPath string, sshAgentSetupErr error) {
+func (m *InstallManager) gatherLogs(provision *hivev1.ClusterProvision, cd *hivev1.ClusterDeployment, sshPrivKeyPath string, sshAgentSetupErr error) {
 	if !m.isBootstrapComplete() {
 		if sshAgentSetupErr != nil {
 			m.log.Warn("unable to fetch logs from bootstrap node as SSH agent was not configured")
@@ -849,14 +868,39 @@ func (m *InstallManager) gatherLogs(cd *hivev1.ClusterDeployment, sshPrivKeyPath
 			m.log.WithError(err).Warn("error fetching logs from bootstrap node")
 			return
 		}
+
 		m.log.Info("successfully gathered logs from bootstrap node")
 	} else {
 		if err := m.gatherClusterLogs(cd); err != nil {
 			m.log.WithError(err).Warn("error fetching logs with oc adm must-gather")
 			return
 		}
+
 		m.log.Info("successfully ran oc adm must-gather")
 	}
+
+	// At this point, all log files are in m.LogsDir
+	// Gather the filenames
+	files, err := ioutil.ReadDir(m.LogsDir)
+	if err != nil {
+		m.log.WithError(err).WithField("clusterprovision", types.NamespacedName{Name: provision.Name, Namespace: provision.Namespace}).Error("error reading Logsdir")
+		return
+	}
+
+	filepaths := []string{}
+	for _, file := range files {
+		if file.IsDir() {
+			m.log.WithField("directory", file.Name()).Debug("not uploading directory")
+			continue
+		}
+
+		filepaths = append(filepaths, filepath.Join(m.LogsDir, file.Name()))
+	}
+
+	uploadErr := m.actuator.UploadLogs(cd.Spec.ClusterName, provision, m.DynamicClient, m.log, filepaths...)
+	if uploadErr != nil {
+		m.log.WithError(uploadErr).Error("error uploading logs")
+	}
 }
 
 func (m *InstallManager) gatherClusterLogs(cd *hivev1.ClusterDeployment) error {
@@ -1047,8 +1091,10 @@ func (m *InstallManager) isBootstrapComplete() bool {
 
 func uploadAdminKubeconfig(provision *hivev1.ClusterProvision, m *InstallManager) (*corev1.Secret, error) {
 	m.log.Infoln("uploading admin kubeconfig")
-	fullPath := filepath.Join(m.WorkDir, adminKubeConfigRelativePath)
 
+	var kubeconfigSecret *corev1.Secret
+
+	fullPath := filepath.Join(m.WorkDir, adminKubeConfigRelativePath)
 	if _, err := os.Stat(fullPath); os.IsNotExist(err) {
 		m.log.WithField("path", fullPath).Error("admin kubeconfig file does not exist")
 		return nil, err
@@ -1060,7 +1106,7 @@ func uploadAdminKubeconfig(provision *hivev1.ClusterProvision, m *InstallManager
 		return nil, err
 	}
 
-	kubeconfigSecret := &corev1.Secret{
+	kubeconfigSecret = &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      fmt.Sprintf(adminKubeConfigSecretStringTemplate, m.ClusterProvisionName),
 			Namespace: m.Namespace,
@@ -1095,23 +1141,36 @@ func uploadAdminKubeconfig(provision *hivev1.ClusterProvision, m *InstallManager
 	return kubeconfigSecret, nil
 }
 
-func uploadAdminPassword(provision *hivev1.ClusterProvision, m *InstallManager) (*corev1.Secret, error) {
-	m.log.Infoln("uploading admin username/password")
-	fullPath := filepath.Join(m.WorkDir, adminPasswordRelativePath)
+func loadAdminPassword(m *InstallManager) (string, error) {
+	m.log.Infoln("loading admin password")
 
+	var password string
+	fullPath := filepath.Join(m.WorkDir, adminPasswordRelativePath)
 	if _, err := os.Stat(fullPath); os.IsNotExist(err) {
 		m.log.WithField("path", fullPath).Error("admin password file does not exist")
-		return nil, err
+		return "", err
 	}
 
 	passwordBytes, err := ioutil.ReadFile(fullPath)
 	if err != nil {
 		m.log.WithError(err).WithField("path", fullPath).Error("error reading admin password file")
-		return nil, err
+		return "", err
 	}
 
 	// Need to trim trailing newlines from the password
-	password := strings.TrimSpace(string(passwordBytes))
+	password = strings.TrimSpace(string(passwordBytes))
+
+	return password, nil
+}
+
+func uploadAdminPassword(provision *hivev1.ClusterProvision, m *InstallManager) (*corev1.Secret, error) {
+	m.log.Infoln("uploading admin username/password")
+
+	// Need to trim trailing newlines from the password
+	password, err := m.loadAdminPassword(m)
+	if err != nil {
+		return nil, err
+	}
 
 	s := &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
